@@ -1,9 +1,3 @@
-'''
-Author: Aaron Levi (aaronlyy)
-Description: levish lets you create your own shell with custom commands.
-Version: 0.1.1
-'''
-
 import os
 from inspect import getfullargspec
 
@@ -11,11 +5,24 @@ from pyfiglet import figlet_format
 
 
 # TODO add system command functionality
+# TODO add some more basic commands (ls, cd, mkdir, rm, etc)
+
+
+
+#* ------------------------
+#* --- Main Shell class ---
+#* ------------------------
 
 class Shell:
-    '''
-    Create new Shell object
-    '''
+    """Creates a new Shell object
+    
+    Args:
+        name (str): Name of the shell.
+        show_cwd (bool, optional): Add current working directory to prefix, defaults to "False"
+        prefix (str, optional): Shell prefix, defaults to "[>]"
+        figlet (bool, optional): Enable figlet print on first start, defaults to "False"
+        figlet_font (str, optional): Change figlet font, defaults to "standard"
+    """
     def __init__(self, name, show_cwd=False, prefix="[>] ", figlet=False, figlet_font="standard"):
         self._name = name
         self._show_cwd = show_cwd
@@ -26,11 +33,17 @@ class Shell:
         self._figlet_font = figlet_font
         self._looping = True
 
-    # --- main loop function ---
+
+
+    #* ---------------------
+    #* --- loop function ---
+    #* ---------------------
+
     def _loop(self):
-        '''
-        main loop waiting for input angel du geile sau
-        '''
+        """Main loop waiting for input and executing functions.
+
+        Loop can be exited by calling "_break_loop()"
+        """
         while self._looping:
             if self._show_cwd:
                 inp = input(f"{self._name}@{os.getcwd()} {self._prefix}")
@@ -54,51 +67,71 @@ class Shell:
             else:
                 # continue loop if inp == 0
                 continue
-    # -------------------------
 
-    # --- add function to object ---
-    def add_command(self, cmd, function, description="no description"):
-        '''
-        Add a new command to the shell.\n
+
+
+    #* ----------------------------
+    #* --- add command function ---
+    #* ----------------------------
+
+    def add_command(self, cmd: "str", function: "func", description: "str"="no description"):
+        """Add a new command to the Shell
+
         Args:
-            cmd (str): the commands
-            function (function): the function that is executed on command call
-            description (str) [opt]: command description 
-        '''
-        # check for naming
+            cmd (str): command that executes the function
+            function (function): functions that is executed when command is typed
+            description (str, optional): Defaults to "no description".
+
+        Raises:
+            CommandAlreadyExistError: Raised when a command is added that already exist in command dict.
+            MissingArgsInFunctionError: Raised when added command function has no parameter called args.
+            FunctionNotCallableError: Raised when added command function is not actually a function.
+        """
         # check if passed in function is really a function
         if callable(function):
             # check if function takes 1 argument called args
             if "args" in getfullargspec(function).args:
-                # check if function does not already exist
                 if not cmd in self._commands:
-                    self._commands[cmd] = {"func": function, "desc": description}
+                    self._commands[cmd] = {
+                        "func": function,
+                        "desc": description
+                    }
                 else:
                     raise CommandAlreadyExistError(cmd)
             else:
                 raise MissingArgsInFunctionError(cmd)
         else:
-            # raise function not callable error
             raise FunctionNotCallableError(cmd)
-    # ------------------------------
 
-    # --- break loop ---
+
+    def command(self, function: "function"):
+        """add_command but as decorator
+        """
+        self.add_command(function.__name__, function, f"{function.__doc__.strip()}")
+
+
+
+    #* --------------------------------------
+    #* --- other internal functions start ---
+    #* --------------------------------------
+
     def _break_loop(self):
-        '''
-        break out of main loop
-        '''
+        """Break out of _loop by setting self._looping to False."""
         self._looping = False
-    # --------------------
 
-    # --- build help function ---
     def _build_help(self):
+        """Builds the help string that is displayed when the 'help' command is executed."""
         self._help = "------------------------\nCOMMAND: DESCRIPTION"
         for cmd in self._commands:
             self._help += f"\n{cmd}: {self._commands[cmd]['desc']}"
         self._help += "\n------------------------"
-    # ---------------------------
 
-    # --- internal commands ---
+
+
+    #* ----------------------
+    #* --- basic commands ---
+    #* ----------------------
+
     def _cmd_help(self, args):
         if len(args) > 0:
             if args[0] in self._commands:
@@ -108,7 +141,6 @@ class Shell:
         else:
             print(self._help)
 
-
     def _cmd_cls(self, args):
         os.system("cls")
     
@@ -117,23 +149,28 @@ class Shell:
 
     def _cmd_exit(self, args):
         self._break_loop()
-    
 
-    def enable_basic_commands(self):
-        '''
-        This adds basic commands like 'cls/clear' & 'exit' to the shell
-        '''
+
+
+
+    #* -----------------------------------
+    #* --- add basic commands function ---
+    #* -----------------------------------
+
+    def add_basic_commands(self):
+        """This adds basic commands like 'cls/clear' & 'exit' to the shell"""
         self.add_command("cls", self._cmd_cls, "Clears the screen (windows)")
         self.add_command("clear", self._cmd_clear, "Clears the screen (unix)")
         self.add_command("exit", self._cmd_exit, "Exit the shell")
-    # ------------------------------
 
-    
-    # --- run function ---
+
+
+    #* --------------------
+    #* --- run function ---
+    #* --------------------
+
     def run(self):
-        '''
-        Start the shell
-        '''
+        """Initialize the Shell and run self._loop"""
         self.add_command("help", self._cmd_help, "Shows help")
         # create help string
         self._build_help()
@@ -142,9 +179,13 @@ class Shell:
             print(figlet_format(self._name, self._figlet_font))
         # start main loop
         self._loop()
-    # ------------------
 
-# --- exception classes ---
+
+
+    #* -------------------------
+    #* --- custom exceptions ---
+    #* -------------------------
+
 class CommandAlreadyExistError(Exception):
     def __init__(self, cmd):
         self.message = f"'{cmd}': This command already exists."
@@ -164,16 +205,4 @@ class FunctionNotCallableError(Exception):
         self.message = f"'{cmd}': The passed in function is not a callable object."
 
     def __str__(self):
-        return self.message
-# ------------------------
-
-
-if __name__ == "__main__":
-
-    def test(args):
-        print("test function")
-    
-    sh = Shell("levish", show_cwd=True, figlet=True, figlet_font="slant")
-    sh.enable_basic_commands()
-    sh.add_command("test", test)
-    sh.run()
\ No newline at end of file
+        return self.message
\ No newline at end of file
